<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

/**
 * Custom Payment Gateway.
 *
 * Provides a Custom Payment Gateway, mainly for testing purposes.
 */
add_action('plugins_loaded', 'init_spc_gateway_class');
function init_spc_gateway_class(){

    class SPC_Gateway_Sprint extends WC_Payment_Gateway {

        public $domain;

        /**
         * Constructor for the gateway.
         */
        public function __construct() {

            if (!session_id()) {
       session_start();
    }
    // echo $_SESSION['tokin'];
    $this->domain = 'sprintcheckout_payment';
    $this->id = 'sprint';
    $this->icon = apply_filters('woocommerce_sprint_gateway_icon', '');
    $this->has_fields = false;
    $this->method_title = __('Sprintcheckout', $this->domain);
    $this->method_description = __('Sprintcheckout General Settings', $this->domain);
    // Load the settings.
    $this->init_form_fields();
    $this->init_settings();
    // Define user set variables
    $this->title = $this->get_option('title');
    $this->description = 'Select token to pay your order';
    $this->order_status = $this->get_option('order_status', 'completed');
    $this->ethereum_address = $this->get_option('ethereum_address');
    // Actions
    add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
    add_action('woocommerce_thankyou_'.$this->id, array($this,'thankyou_page'));
    // Sprinter Emails
    add_action('woocommerce_email_before_order_table',array($this, 'email_instructions'), 10, 3);
    add_action('wp_enqueue_scripts', array($this, 'spc_enqueue_script_front'));
    add_action('admin_enqueue_scripts', array($this, 'admin_enqueue_script_front'));
    add_action('woocommerce_api_{webhook name}', array($this, 'webhook'));
        
        }


/**
* Initialise Gateway Settings Form Fields.
*/
public function init_form_fields()
{

$this->form_fields = array(
//        'enabled' => array(
//            'title' => __('ZkSync Payments Enable/Disable', $this->domain) ,
//            'type' => 'checkbox',
//            'label' => __('Enable Sprint Payment', $this->domain) ,
//            'default' => 'yes',
//            'class' => 'zksync_pay_title'
//        ),
        'title' => array(
            'title'       => __( 'Title', $this->domain ),
            'type'        => 'text',
            'description' => __( 'This controls the title which the user sees during checkout.', $this->domain ),
            'default'     => __( 'Western Union Payment', $this->domain ),
            'desc_tip'    => true,
        ),
//        'order_status' => array(
//                    'title'       => __( 'Order Status', $this->domain ),
//                    'type'        => 'select',
//                    'class'       => 'wc-enhanced-select',
//                    'description' => __( 'Choose whether status you wish after checkout.', $this->domain ),
//                    'default'     => 'wc-processing',
//                    'desc_tip'    => true,
//                    'options'     => wc_get_order_statuses()
//                ),
                'mainnet' => array(
                    'title' => __('Mainnet', $this->domain),
                    'type' => 'checkbox',
                    'label' => __('Mainnet', $this->domain),
                    'class' => 'network',
                    'default' => 'no'
                ),
                'rinkeby' => array(
                    'title' => __('Rinkeby', $this->domain),
                    'type' => 'checkbox',
                    'label' => __('Rinkeby', $this->domain),
                    'class' => 'network',
                    'default' => 'no'
                ),
                'ropsten' => array(
                    'title' => __('Ropsten', $this->domain),
                    'type' => 'checkbox',
                    'label' => __('Ropsten', $this->domain),
                    'class' => 'network',
                    'default' => 'no'
                ),
                'eth_address' => array(
                    'title' => __('Your Ethereum Public Address', $this->domain),
//                     'title' => __('Your Ethereum Public Address (<a href="https://docs.sprintcheckout.com/merchants/zksync-account-activation" target="_blank">Activate on zkSync</a>)', $this->domain),
                    'type' => 'text',
                    'label' => __('Your Ethereum Public Address', $this->domain),
                    'class' => 'eth_address'
//                    'custom_attributes' => array('readonly' => 'readonly'),
                ),
                'customer_key' => array(
                    'title' => __('Woocommerce API key', $this->domain),
                    'type' => 'text',
                    'label' => __('Woocommerce API key', $this->domain),
                    'placeholder' => _x('Woocommerce> Settings> Rest API> Add Key(Read/Write)', 'placeholder', 'woocommerce'),
                    'class' => 'customer_key'
                ),
                'customer_secret' => array(
                    'title' => __('Woocommerce API secret', $this->domain),
                    'type' => 'text',
                    'label' => __('Woocommerce API secret', $this->domain),
                    'placeholder' => _x('Woocommerce> Settings> Rest API> Add Key(Read/Write)', 'placeholder', 'woocommerce'),
                    'class' => 'customer_secret'
                ),
                'title' => array(
                'type' => 'hidden',
                'default' => __('Sprintcheckout via ZkSync', $this->domain),
                ),
    );

}

function admin_options()
{
?>

<div class="sprint_checkout" style="background: #fff;padding: 10px 20px;box-shadow: 2px 2px 5px #ddd, -2px -2px 5px #ddd;border-radius: 3px;">

    <h1>Sprint Checkout <small class="wc-admin-breadcrumb">
        <a href="<?php echo site_url(); ?>/wp-admin/admin.php?page=wc-settings&amp;tab=checkout" aria-label="Return to payments">⤴</a></small>
    </h1><hr>
    
    <div id="sprintcheckout_support_request"><p>Email us with any questions or issues to support@sprintcheckout.com</p></div>

    <table class="form-table zksync">
    <?php $this->generate_settings_html(); ?>
    <tr>
        <td style="font-weight: bold;"> ZkSync Accepted Tokens </td>
    </tr>
    <tr class="token_row">
        <th>Select all<input style="margin-left: 4px" type='checkbox' id="accept-all-tokens" name='accept-all-tokens' class='accept' value=''></th>
        <th>Token Symbol</th>
        <th>Contract Address</th>
    </tr>
    <?php
    $url = "https://api.zksync.io/api/v0.1/tokens";
    $response = wp_remote_get( $url );
    $data = json_decode($response['body']);
    global $wpdb;
    $table_name = $wpdb->base_prefix . 'spc_tokins';
    //$mytokins = $wpdb->get_results("SELECT * from $table_name WHERE id=1");
    $mytokins = $wpdb->get_row("SELECT * FROM $table_name WHERE id=1");
    $final = unserialize($mytokins->tokin);
    global $wp;
    //print_r($final);
    if (!empty($data))
    {
        foreach ($data as $d)
        {
            echo "<tr class='token_row'>";
            if (array_search($d->symbol, array_column($final, 'csign')) !== false)
            {
                echo "<td><input type='checkbox' name='accept' class='accept' value='" . esc_attr($d->symbol) . "' checked></td>";
            }
            else
            {
                echo "<td><input type='checkbox' name='accept' class='accept' value='" . esc_attr($d->symbol) . "'></td>";
            }
            echo "<td>".esc_attr($d->symbol)."</td>";
            echo "<td>".esc_attr($d->address)."</td>";
            echo "</tr>";
        }
    }
    else
    {
        echo "<tr class='token_row' style='background:#fff; border-bottom:1px solid #ddd;text-align: center;'>";
        echo "<td colspan='3' style='color:red;'> API ERROR </td>";
        echo "</tr>";
    } ?>
<!--    <tr>-->
<!--        <td style="font-weight: bold;padding-top: 20px; padding-bottom: 0px;"> Store Currency </td>-->
<!--    </tr>-->
<!---->
<!--    <tr>-->
<!--        <td><input type="text" name="store_currency" name="store_currency" value="--><?php //echo get_woocommerce_currency_symbol(); ?><!--" style="width:210px;" readonly>-->
<!--        </td> -->
<!--        <td><i>You can change this in your Woocommerce General Settings</i></td>-->
<!--    </tr>-->
    <?php
    echo "</table>";

echo "</div>";

}

        /**
         * Output for the order received page.
         */
        public function thankyou_page() {
            if ( $this->instructions )
                echo wpautop(wptexturize(sanitize_text_field($this->instructions)));
        }

        /**
         * Add content to the WC emails.
         *
         * @access public
         * @param WC_Order $order
         * @param bool $sent_to_admin
         * @param bool $plain_text
         */
        public function email_instructions( $order, $sent_to_admin, $plain_text = false ) {
            if ( $this->instructions && ! $sent_to_admin && 'spc' === $order->payment_method && $order->has_status( 'on-hold' ) ) {
                echo wpautop( wptexturize( sanitize_text_field($this->instructions)) ) . PHP_EOL;
            }
        }

        public function payment_fields()
        {
            if ( $description = $this->get_description() ) {
                    echo wpautop( wptexturize( sanitize_text_field($description) ) );
            }
                //echo  $this->method_description;
                //echo $this->method_title;
                global $wpdb;
                $table_name = $wpdb->base_prefix . 'spc_tokins';
                $mytokins = $wpdb->get_results("SELECT * from $table_name");
                $allowed_tokens = unserialize($mytokins[0]->tokin);


                $url = "https://sprintcheckout-mvp.herokuapp.com/checkout/v1/tokens/rates?layer=zksync&amount=" . WC()->cart->total . "&store_currency=".get_woocommerce_currency()."&site_url=" . get_site_url();

                $response = wp_remote_get( $url );
                $data = json_decode($response['body']);

            $twoDecimalsTokens = array("busd", "dai", "ausdc", "eurs", "gusd", "susd", "tusd", "usdc", "usdt");

                echo "<select>";
                foreach ($data -> tokenPrices as $tokenTicker => $tokenPrice)
                {
                    if (in_array($tokenTicker, $twoDecimalsTokens)) {
                        echo "<option>" . esc_attr(strtoupper($tokenTicker)) . " - " . round(esc_attr($tokenPrice), 2) . "</option>";
                    } else {
                        echo "<option>" . esc_attr(strtoupper($tokenTicker)) . " - " . round(esc_attr($tokenPrice), 6) . "</option>";
                    }
//                    echo "<option>" . $info['csign'] . ' ' . get_woocommerce_currency_symbol() . '' . $info['amount'] . "</option>";
                }
                echo "<select>";

        }

        /**
         * Process the payment and return the result.
         *
         * @param int $order_id
         * @return array
        */
        public function process_payment($order_id)
        {
            $_SESSION['amount'] = WC()->cart->total;
            $payment_url = "https://pay.sprintcheckout.com/?order_id=" .urlencode(base64_encode($order_id))
                . "&layer" . "=" . urlencode(base64_encode("zksync"))
                . "&token" . "=" . urlencode(base64_encode($_SESSION['tokin']))
                . "&amount" . "=" . urlencode(base64_encode($_SESSION['amount']))
                . "&store_currency" . "=" .  urlencode(base64_encode(get_woocommerce_currency()))
                . "&site_url" . "=" . urlencode(base64_encode(get_site_url()));

            return array(
                'result' => 'success',
                //  'redirect' => $this->get_return_url($payment_url)
                'redirect' => $payment_url
            );
        } // Process Payment
    }
}

add_filter( 'woocommerce_payment_gateways', 'add_spc_gateway_class' );
function add_spc_gateway_class( $methods ) {
    $methods[] = 'SPC_Gateway_Sprint'; 
    return $methods;
}
